﻿
qx.Mixin.define("wisej.MMovablePatch", {

	members: {

		/**
		 * Computes the new drag coordinates
		 *
		 * @param e {qx.event.type.Pointer} Pointer event
		 * @return {Map} A map with the computed drag coordinates
		 */
		__computeMoveCoordinates: function (e) {
			var range = this.__dragRange;
			var pointerLeft = Math.max(range.left, Math.min(range.right, e.getDocumentLeft()));
			var pointerTop = Math.max(range.top, Math.min(range.bottom, e.getDocumentTop()));

			var viewportLeft = this.__dragLeft + pointerLeft;
			var viewportTop = this.__dragTop + pointerTop;

			// @ITG:Wisej: Adjust the element's coordinates to the possible element's CSS rotation.
			var adjustTop = 0;
			var adjustLeft = 0;
			var domEl = this.getContentElement().getDomElement();
			if (domEl) {
				var clientRect = domEl.getBoundingClientRect();
				adjustTop = parseInt(domEl.style.top) - clientRect.top;
				adjustLeft = parseInt(domEl.style.left) - clientRect.left;
			}

			// @ITG:Wisej: Added support for keeping the movable widget wholly in the view area.
			if (this.isKeepOnScreen()) {
				var parentLeft = parseInt(viewportLeft - this.__parentLeft, 10) + adjustLeft;
				var parentTop = parseInt(viewportTop - this.__parentTop, 10) + adjustTop;

				var windowBounds = this.getBounds();
				var containerBounds = this.getLayoutParent().getBounds();

				if (parentLeft < 0)
					parentLeft = 0;
				else if (parentLeft + windowBounds.width > containerBounds.width)
					parentLeft = parentLeft - (parentLeft + windowBounds.width - containerBounds.width);

				if (parentTop < 0)
					parentTop = 0;
				else if (parentTop + windowBounds.height > containerBounds.height)
					parentTop = parentTop - (parentTop + windowBounds.height - containerBounds.height);

				return {
					viewportLeft: parseInt(viewportLeft, 10),
					viewportTop: parseInt(viewportTop, 10),
					parentLeft: parentLeft,
					parentTop: parentTop
				};
			}

			return {
				viewportLeft: parseInt(viewportLeft, 10),
				viewportTop: parseInt(viewportTop, 10),

				parentLeft: parseInt(viewportLeft - this.__parentLeft, 10) + adjustLeft,
				parentTop: parseInt(viewportTop - this.__parentTop, 10) + adjustTop
			};
		},
	}
});

qx.Class.patch(wisej.web.Label, wisej.MMovablePatch);